@@ -6,7 +6,7 @@
 #
 # Copyright (c) 2008-2009 Yu Yuwei <acevery@gmail.com>
 # Copyright (c) 2009-2014 Caius "kaio" CHANCE <me@kaio.net>
-# Copyright (c) 2012-2015 Mike FABIAN <mfabian@redhat.com>
+# Copyright (c) 2012-2015, 2021 Mike FABIAN <mfabian@redhat.com>
 # Copyright (c) 2019      Peng Wu <alexepico@gmail.com>
 #
 # This library is free software; you can redistribute it and/or
@@ -19,10 +19,11 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-#
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>
+'''
+Program to create sqlite databases from the table sources
+'''
 
 from typing import Tuple
 from typing import List
@@ -62,7 +63,7 @@ class InvalidTableName(Exception):
     Raised when an invalid table name is given
     """
     def __init__(self, name) -> None:
-        super(InvalidTableName, self).__init__()
+        super().__init__()
         self.table_name = name
 
     def __str__(self) -> str:
@@ -172,7 +173,41 @@ if not _OPTIONS.name:
         + 'contains the source of the IME!')
     sys.exit(2)
 
+
+class Section:
+    '''Helper class for parsing the sections of the tables marked
+    with BEGIN_* and END_*.
+    '''
+    patt: re.Pattern
+    start: str
+    end: str
+    in_section: bool
+
+    def __init__(self, patt: re.Pattern, start: str, end: str):
+        self.patt = patt
+        self.start = start.strip()
+        self.end = end.strip()
+        self.in_section = False
+
+    def match(self, line: str) -> bool:
+        '''
+        Returns True if the line is inside the section and matches
+        the pattern of the section.
+        '''
+        if self.in_section:
+            if self.end == line.strip():
+                self.in_section = False
+            elif self.patt.match(line):
+                return True
+        elif self.start == line.strip():
+            self.in_section = True
+
+        return False
+
+
 def main():
+    '''Main program'''
+
     def debug_print(message: str) -> None:
         if _OPTIONS.debug:
             print(message)
@@ -180,7 +215,7 @@ def main():
     if not _OPTIONS.only_index:
         try:
             os.unlink(_OPTIONS.name)
-        except:
+        except Exception:
             pass
 
     debug_print('Processing Database')
@@ -188,9 +223,11 @@ def main():
                                  user_db=None,
                                  create_database=True)
 
-    def parse_source(f: Iterable[str]) -> Tuple[List[str], List[str], List[str]]:
+    def parse_source(
+            f: Iterable[str]) -> Tuple[List[str], List[str], List[str]]:
         _attri: List[str] = []
         _table: List[str] = []
+        _table_extra: List[str] = []
         _gouci: List[str] = []
         patt_com = re.compile(r'^###.*')
         patt_blank = re.compile(r'^[ \t]*$')
@@ -198,13 +235,23 @@ def main():
         patt_table = re.compile(r'([^\t]+)\t([^\t]+)\t([0-9]+)(\t.*)?$')
         patt_gouci = re.compile(r' *[^\s]+ *\t *[^\s]+ *$')
 
+        sec_conf = Section(
+            patt_conf, "BEGIN_DEFINITION", "END_DEFINITION")
+        sec_table = Section(
+            patt_table, "BEGIN_TABLE", "END_TABLE")
+        sec_table_extra = Section(
+            patt_table, "BEGIN_TABLE_EXTRA", "END_TABLE_EXTRA")
+        sec_gouci = Section(
+            patt_gouci, "BEGIN_GOUCI", "END_GOUCI")
+
         for line in f:
             if (not patt_com.match(line)) and (not patt_blank.match(line)):
-                for _patt, _list in (
-                        (patt_table, _table),
-                        (patt_gouci, _gouci),
-                        (patt_conf, _attri)):
-                    if _patt.match(line):
+                for _sec, _list in (
+                        (sec_table, _table),
+                        (sec_table_extra, _table_extra),
+                        (sec_gouci, _gouci),
+                        (sec_conf, _attri)):
+                    if _sec.match(line):
                         _list.append(line)
                         break
 
@@ -241,6 +288,7 @@ def main():
                 _gouci.append('%s\t%s' %(key, gouci_dict[key]))
             _gouci.sort()
 
+        _table += _table_extra
         return (_attri, _table, _gouci)
 
     def parse_pinyin(f: Iterable[str]) -> List[str]:
@@ -316,7 +364,7 @@ def main():
         for line in f:
             try:
                 attr, val = line.strip().split('=')
-            except:
+            except Exception:
                 attr, val = line.strip().split('==')
             attr = attr.strip().lower()
             val = val.strip()
